#include <openssl/bn.h>
#include <openssl/ec.h>
#include <signal.h>
#include <assert.h>

int main(int argc, char **argv) {
    BIGNUM *a, *p, *b, *x;
    BN_CTX *ctx;
    BN_CTX *ctx2;
    EC_GROUP *g;
    EC_POINT *pt;

    ctx = BN_CTX_new();
    ctx2 = BN_CTX_new();

    BN_CTX_start(ctx);
    a = BN_CTX_get(ctx);
    b = BN_CTX_get(ctx);
    p = BN_CTX_get(ctx);
    x = BN_CTX_get(ctx);

    // A composite!
    BN_set_word(p, 3601);
    BN_set_word(x, 3599);

    // Note: I think you can use any curve...
    BN_one(a);
    BN_one(b);

    g = EC_GROUP_new_curve_GFp(p, a, b, NULL);

    // I know that BN_mod_sqrt(p, 1109) will hang because p is composite, but
    // EC_POINT_set_compressed_coordinates actually calls BN_mod_sqrt(p, (x^3
    // + ax + b) mod p). Therefore, I need to know a good value for x...
    //
    // Mathematica 12.0.0 Kernel for Linux x86 (64-bit)
    // In[3]:= FindInstance[1109 == Mod[x ^ 3 + a*x + b, 3601], {x}, PositiveIntegers]
    // Out[3]= {{x -> 277}}
    BN_set_word(x, 277);

    // This is effectively BN_mod_sqrt(p, (x^3 + ax + b) mod p)
    pt = EC_POINT_new(g);
    EC_POINT_set_compressed_coordinates(g, pt, x, 1, NULL);

    // Okay, so encoding that into some EC params...
    //
    // -----BEGIN EC PARAMETERS-----
    // MCICAQEwDQYHKoZIzj0BAQICDhEwBgQBAQQBAQQDAwEVAgEB
    // -----END EC PARAMETERS-----
    //
    // And then stuffing those params into a public key....
    //
    // -----BEGIN PUBLIC KEY-----
    // MDIwLQYHKoZIzj0CATAiAgEBMA0GByqGSM49AQECAg4RMAYEAQEEAQEEAwMBFQIB
    // AQMBAA==
    // -----END PUBLIC KEY-----
    //
    // Now we can finally stuff that key into a certificate...
    //
    // -----BEGIN CERTIFICATE-----
    // MIGmMIGRoAMCAQICAQAwDQYJKoZIhvcNAQELBQAwETEPMA0GA1UEAwwGdGF2aXNv
    // MB4XDTIxMDMyNTAwMDAwMFoXDTIzMDMzMDIzNTk1OVowETEPMA0GA1UEAwwGdGF2
    // aXNvMDIwLQYHKoZIzj0CATAiAgEBMA0GByqGSM49AQECAg4RMAYEAQEEAQEEAwMB
    // FQIBAQMBADANBgkqhkiG9w0BAQsFAAMBAA==
    // -----END CERTIFICATE-----

    EC_POINT_free(pt);
    EC_GROUP_free(g);
    BN_CTX_end(ctx);
    BN_CTX_free(ctx);
    return 0;
}

